using System;
using UnityEngine;
using UnityEngine.UI;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using MVR.FileManagementSecure;

namespace ky1001
{
    public class AddAtom : MVRScript
    {
        #region PluginInfo
        public string pluginAuthor = "ky1001";
        public string pluginName = "AddAtom";
        public string pluginVersion = "7.0";
        public string pluginDate = "2022-11-18";
        public string pluginDescription = "";
        #endregion

        #region Var
        private Atom AddedAtom;
        private Atom currentAtom;
        private Atom targetAtom;
        private Atom arrivalAtom;
        private UIDynamic spacer;
        private JSONStorableFloat maxD;
        private UIDynamicSlider smaxD;
        private JSONStorableBool cr_Bool;
        private JSONStorableBool ts_Bool;
        private UIDynamicToggle ts_Toggle;
        private JSONStorableBool ja_Bool;
        private UIDynamicToggle ja_Toggle;
        private JSONStorableBool abp_Bool;
        private UIDynamicToggle abp_Toggle;
        private JSONStorableBool mark_Bool;
        private UIDynamicToggle mark_Toggle;
        private JSONStorableBool kh_Bool;
        private UIDynamicToggle kh_Toggle;
        private Vector3 currentAtomPosition;
        private Vector3 targetAtomPosition;
        private Vector3 bringAtomPosition;
        private UIDynamicButton loadPresetUI;
        private UIDynamicButton add_button;
        private UIDynamicButton bi_button;
        private UIDynamicButton col_button;
        private UIDynamicButton rfl_button;
        protected JSONStorableStringChooser catJSON;
        protected JSONStorableStringChooser itemJSON;
        protected JSONStorableStringChooser targetJSON;
        protected JSONStorableStringChooser arrivalJSON;
        protected UIDynamicPopup dp1;
        protected UIDynamicPopup dp2;
        protected UIDynamicPopup dp3;
        protected UIDynamicPopup dp4;
        private bool add;
        public List<List<string>> AtomList = new List<List<string>>();
        public List<string> CatList = new List<string>();
        protected JSONStorableString filename;
        private string iniFile;
        private string iniFilePath;

        private string str1;
        private string str2;

        private GameObject line = null;
        private GameObject line2 = null;
        private GameObject sphere = null;
        private GameObject sphereY = null;
        private GameObject sphereG = null;
        private Vector3 kh;
        private float mkSize = 0.5f;
        private bool mkUpDown = true;

        private JSONStorableBool wait_Bool;
        private UIDynamicToggle wait_Toggle;

        private JSONStorableBool _isCopyRotation;
        private Quaternion currentAtomRotation;
        #endregion      

        //public string[] CatList = new string[]
        public string[] Items = new string[] {
            "Animation",
            //"Environments",
            //"FloorsAndWalls",
            "Force",
            //"Furniture",
            "Light",
            "Misc",
            //"People",
            //"Props",
            //"Reflective",
            //"Shapes",
            //"Sound",
            "Toys",
            "Triggers"
        };

        private const string loadExt = "json";
        private const string presetLoadPath = "Saves/PluginData/AddAtom";
        public string LoadData;

        public override void Init()
        {
            try
            {
                cr_Bool = new JSONStorableBool("Bring It to Player", false);
                RegisterBool(cr_Bool);

                ja_Bool = new JSONStorableBool("Just Add", false);
                RegisterBool(ja_Bool);
                ja_Toggle = CreateToggle(ja_Bool, false);
                ja_Toggle.toggle.onValueChanged.AddListener(delegate
                {
                    if (ja_Bool.val == true)
                    {
                        abp_Bool.val = false;
                        kh_Bool.val = false;
                        targetJSON.val = "None";
                    }
                });

                ts_Bool = new JSONStorableBool("Target Select after Add/Move", false);
                RegisterBool(ts_Bool);
                ts_Toggle = CreateToggle(ts_Bool, false);

                col_button = CreateButton("Target Collision On", false);
                if (col_button != null)
                {
                    col_button.button.onClick.AddListener(collisionOn);
                }
                JSONStorableAction collisionOnAct = new JSONStorableAction("Target Collision On", () => { collisionOn(); });
                RegisterAction(collisionOnAct);

                filename = new JSONStorableString("AddAtomPreset Filename", "Saves/PluginData/AddAtom/");
                RegisterString(filename);

                spacer = CreateSpacer(false);
                spacer.height = 5f;

                add_button = CreateButton("Add Atom");
                add_button.buttonColor = Color.green;
                if (add_button != null)
                {
                    add_button.button.onClick.AddListener(addAtom);
                }
                JSONStorableAction AddAtomAct = new JSONStorableAction("Add Atom", () => { addAtom(); });
                RegisterAction(AddAtomAct);

                loadPresetUI = CreateButton("Load Preset");
                loadPresetUI.button.onClick.AddListener(() =>
                {
                    filename.val = "";
                    FileManagerSecure.CreateDirectory(presetLoadPath);
                    var shortcuts = FileManagerSecure.GetShortCutsForDirectory(presetLoadPath);
                    SuperController.singleton.GetMediaPathDialog(HandleLoadPreset, "json", presetLoadPath, false, true, false, null, false, shortcuts);
                    loadPresetUI.label = "Please Wait";
                    loadPresetUI.buttonColor = Color.red;
                    loadPresetUI.textColor = Color.white;
                    Invoke("mkCatList", 2f);
                });
                FileManagerSecure.CreateDirectory(presetLoadPath);

                kh_Bool = new JSONStorableBool("Keep Height", false);
                RegisterBool(kh_Bool);
                kh_Toggle = CreateToggle(kh_Bool, true);
                kh_Toggle.toggle.onValueChanged.AddListener(delegate
                {
                    if (kh_Bool.val == true)
                    {
                        ja_Bool.val = false;
                    }
                });

                abp_Bool = new JSONStorableBool("Add/Move to Zero", false);
                RegisterBool(abp_Bool);
                abp_Toggle = CreateToggle(abp_Bool, true);
                abp_Toggle.toggle.onValueChanged.AddListener(delegate
                {
                    if (abp_Bool.val == true)
                    {
                        ja_Bool.val = false;
                    }
                });

                mark_Bool = new JSONStorableBool("Show Marker", false);
                RegisterBool(mark_Bool);
                mark_Toggle = CreateToggle(mark_Bool, true);

                spacer = CreateSpacer(true);
                spacer.height = 5f;

                bi_button = CreateButton("Bring It", true);
                bi_button.buttonColor = Color.green;
                if (bi_button != null)
                {
                    bi_button.button.onClick.AddListener(BringItNow);
                }
                JSONStorableAction BringItNowAct = new JSONStorableAction("Bring It", () => { BringItNow(); });
                RegisterAction(BringItNowAct);

                rfl_button = CreateButton("Refresh List", true);
                if (rfl_button != null)
                {
                    rfl_button.button.onClick.AddListener(mkList);
                }
                JSONStorableAction refreshListAct = new JSONStorableAction("Refresh List", () => { mkList(); });
                RegisterAction(refreshListAct);

                mkArray();

                iniFile = "";
                var iniFileini = presetLoadPath + "/inifile.ini";
                if (FileManagerSecure.FileExists(iniFileini))
                {
                    iniFile = SuperController.singleton.ReadFileIntoString(iniFileini);
                }

                if (iniFile.Trim() != "")
                {
                    iniFilePath = presetLoadPath + "/" + iniFile.Trim();
                    if (FileManagerSecure.FileExists(iniFilePath))
                    {
                        HandleLoadPreset(iniFilePath);
                    }
                }

                catJSON = new JSONStorableStringChooser("Category", null, "None", "Category", SyncAtom);
                RegisterStringChooser(catJSON);
                dp1 = CreateScrollablePopup(catJSON, false);
                dp1.popupPanelHeight = 650f;
                dp1.popup.selectColor = new Color(0.8f, 0.8f, 0.8f);

                itemJSON = new JSONStorableStringChooser("Atom", null, "None", "Atom");
                RegisterStringChooser(itemJSON);
                dp2 = CreateScrollablePopup(itemJSON, false);
                dp2.popup.selectColor = new Color(0.8f, 0.8f, 0.8f);
                dp2.popupPanelHeight = 550f;

                mkCatList();

                targetJSON = new JSONStorableStringChooser("Target", null, "None", "Target", ClearJustAdd);
                RegisterStringChooser(targetJSON);
                dp3 = CreateScrollablePopup(targetJSON, true);
                dp3.popupPanelHeight = 650f;
                dp3.popup.selectColor = new Color(0.8f, 0.8f, 0.8f);
                dp3.transform.Find("Background").GetComponent<Image>().color = Color.magenta;
                //dp3.popup.onOpenPopupHandlers += ClearJustAdd;                

                arrivalJSON = new JSONStorableStringChooser("Bring to", null, "None", "Bring to");
                RegisterStringChooser(arrivalJSON);
                dp4 = CreateScrollablePopup(arrivalJSON, true);
                dp4.popupPanelHeight = 550f;
                dp4.popup.selectColor = new Color(0.8f, 0.8f, 0.8f);
                dp4.labelTextColor = Color.black;
                dp4.transform.Find("Background").GetComponent<Image>().color = Color.cyan;

                mkList();

                maxD = new JSONStorableFloat("Distance", 1f, 0f, 5f, true);
                smaxD = CreateSlider(maxD, true);
                RegisterFloat(maxD);
                //smaxD.slider.GetComponent<Image>().color = Color.yellow;
                smaxD.transform.Find("Panel").GetComponent<Image>().color = Color.yellow;
                //SuperController.LogMessage($"{nameof(AddAtom)} initialized");

                wait_Bool = new JSONStorableBool("Wait", true);
                RegisterBool(wait_Bool);
                wait_Toggle = CreateToggle(wait_Bool, true);

                _isCopyRotation = new JSONStorableBool("Copy Rotation to Target", false);
                RegisterBool(_isCopyRotation);
                CreateToggle(_isCopyRotation, true);
            }
            catch (Exception e)
            {
                SuperController.LogError($"{nameof(AddAtom)}.{nameof(Init)}: {e}");
            }
        }

        protected void ClearJustAdd(string dummy)
        {
            if (targetJSON.val != "None")
            {
                ja_Bool.val = false;
            }
        }

        public void Start()
        {
            if (containingAtom.name != "CoreControl")
            {
                arrivalJSON.val = containingAtom.name;
            }
            else
            {
                arrivalJSON.val = "Player";
            }
        }

        public void mkCatList()
        {
            List<string> catChoices = new List<string>();
            foreach (string category in CatList)
            {
                catChoices.Add(category);
            }
            catJSON.choices = catChoices;
            catJSON.val = "None";
            loadPresetUI.label = "Load Preset";
            loadPresetUI.buttonColor = col_button.buttonColor;
            loadPresetUI.textColor = Color.black;
        }

        protected void SyncAtom(string category)
        {
            List<string> itemChoices = new List<string>();
            if (category != null)
            {
                if (catJSON.val != "None")
                {
                    var i = 0;
                    foreach (string dummy in CatList)
                    {
                        if (category == AtomList[i][0])
                        {
                            foreach (string item in AtomList[i])
                            {
                                itemChoices.Add(item);
                            }
                        }
                        ++i;
                    }
                }
                itemChoices.RemoveAt(0);
                itemJSON.choices = itemChoices;
                itemJSON.val = "None";
            }
        }

        public void addAtom()
        {
            if (itemJSON.val != "None")
            {
                SuperController.singleton.AddAtomByType(itemJSON.val, true, true, true);
                add = true;
                Invoke("BringItNow", 1f);
            }
        }

        public void BringItNow()
        {
            if (targetJSON.val != "None" || add == true)
            {
                if (add)
                {
                    targetAtom = SuperController.singleton.GetSelectedAtom();
                    AddedAtom = targetAtom;
                }
                else
                {
                    targetAtom = SuperController.singleton.GetAtomByUid(targetJSON.val);
                }
                targetAtomPosition = targetAtom.GetStorableByID("control").transform.position;

                if (cr_Bool.val || arrivalJSON.val == "Player")
                {
                    currentAtomPosition = SuperController.singleton.lookCamera.transform.position;
                    currentAtomRotation = targetAtom.GetStorableByID("control").transform.rotation;
                }
                else if (arrivalJSON.val == "Zero")
                {
                    currentAtomPosition = new Vector3(0f, 0f, 0f);
                    currentAtomRotation = Quaternion.Euler(Vector3.zero);
                }
                else
                {
                    currentAtom = SuperController.singleton.GetAtomByUid(arrivalJSON.val);
                    currentAtomPosition = currentAtom.GetStorableByID("control").transform.position;
                    currentAtomRotation = currentAtom.GetStorableByID("control").transform.rotation;
                }
                Vector3 targetDistance;
                targetDistance = targetAtomPosition - currentAtomPosition;

                if (!(ja_Bool.val && add == true))
                {
                    targetAtom.collisionEnabledJSON.val = false;
                    if (wait_Bool.val == false)
                    {
                        if (targetAtom.type == "Person")
                        {
                            targetAtom.tempDisableRender = false;
                        }
                        else
                        {
                            targetAtom.GetBoolJSONParam("on").val = true;
                        }
                    }
                    else
                    {
                        if (targetAtom.type == "Person")
                        {
                            targetAtom.tempDisableRender = true;
                        }
                        else
                        {
                            targetAtom.GetBoolJSONParam("on").val = false;
                        }
                    }
                    //targetAtom.GetBoolJSONParam("on").val = false;                    
                    var maxDistance = Mathf.Max(Mathf.Abs(targetDistance.x), Mathf.Abs(targetDistance.y), Mathf.Abs(targetDistance.z));
                    if (maxDistance > maxD.val)
                    {
                        targetDistance.x = (targetDistance.x / maxDistance) * maxD.val;
                        targetDistance.y = (targetDistance.y / maxDistance) * maxD.val;
                        targetDistance.z = (targetDistance.z / maxDistance) * maxD.val;
                        targetAtom.GetStorableByID("control").transform.position = currentAtomPosition + targetDistance;
                    }

                    if (abp_Bool.val)
                    {
                        targetAtom.GetStorableByID("control").transform.position = Vector3.zero;
                    }

                    if (kh_Bool.val)
                    {
                        kh = targetAtom.GetStorableByID("control").transform.position;
                        kh.y = targetAtomPosition.y;
                        targetAtom.GetStorableByID("control").transform.position = kh;
                    }

                    if (_isCopyRotation.val)
                    {
                        targetAtom.GetStorableByID("control").transform.rotation = currentAtomRotation;
                    }
                }

                if (add)
                {
                    mkList();
                    targetAtom = AddedAtom;
                    targetJSON.val = targetAtom.name;
                }
                if (wait_Bool.val == false)
                {
                    ReDisplay();
                }
                else
                {
                    Invoke("ReDisplay", 1f);
                }
            }
        }

        public void ReDisplay()
        {
            if (targetAtom.type == "Person")
            {
                targetAtom.tempDisableRender = false;
            }
            else
            {
                targetAtom.GetBoolJSONParam("on").val = true;
            }
            add = false;
            if (ts_Bool.val)
            {
                SuperController.singleton.SelectController(targetAtom.mainController);
            }
            else
            {
                if (containingAtom.name != "CoreControl")
                {
                    SuperController.singleton.SelectController(containingAtom.mainController);
                }
            }
        }

        public void collisionOn()
        {
            if (targetJSON.val != "None")
            {
                targetAtom = SuperController.singleton.GetAtomByUid(targetJSON.val);
                targetAtom.collisionEnabledJSON.val = true;
            }
        }

        public void mkList()
        {
            List<string> targetChoices = new List<string>();
            targetChoices.Add("None");
            foreach (string atomUID in SuperController.singleton.GetAtomUIDs())
            {
                targetAtom = SuperController.singleton.GetAtomByUid(atomUID);
                if (targetAtom.hidden == false && targetAtom.name != "[CameraRig]" && targetAtom.name != "CoreControl")
                {
                    targetChoices.Add(atomUID);
                }
            }
            targetJSON.choices = targetChoices;
            targetJSON.val = "None";

            List<string> arrivalChoices = new List<string>();
            arrivalChoices.Add("Player");
            arrivalChoices.Add("Zero");
            foreach (string atomUID in SuperController.singleton.GetAtomUIDs())
            {
                arrivalAtom = SuperController.singleton.GetAtomByUid(atomUID);
                if (arrivalAtom.hidden == false && arrivalAtom.name != "[CameraRig]" && arrivalAtom.name != "CoreControl")
                {
                    arrivalChoices.Add(atomUID);
                }
            }
            arrivalJSON.choices = arrivalChoices;
        }

        public void mkArray()
        {
            CatList = new List<string>(Items);

            AtomList.Add(new List<string>(new[] {
                "Animation",
                "AnimationPattern"
            }));

            AtomList.Add(new List<string>(new[] {
                "Force",
                "CycleForce",
                "GrabPoint",
                "RhythmForce",
                "SyncForce"
            }));

            AtomList.Add(new List<string>(new[] {
                "Light",
                "AptLamp",
                "AptOutdoorLight",
                "CyberpunkLight",
                "CyberpunkWallLight01",
                "CyberpunkWallLight02",
                "InvisibleLight",
                "ModernRoomLargeLamp",
                "TechnoLight",
                "TechnoLightBar+Light",
                "TechnoNeonCircle+Light",
                "TechnoNeonHeart+Light",
                "TechnoNeonSquare+Light",
                "TechnoNeonTriangle+Light",
                "TechnoRingLight+LIght",
                "Torch"
            }));

            AtomList.Add(new List<string>(new[] {
                "Misc",
                "CustomUnityAsset",
                "Empty",
                "WebBrowser",
                "WebPanel"
            }));

            AtomList.Add(new List<string>(new[] {
                "Toys",
                "Dildo",
                "Paddle",
                "ToyAH",
                "ToyBP"
            }));

            AtomList.Add(new List<string>(new[] {
                "Triggers",
                "Button",
                "CollisionTrigger",
                "LookAtTrigger",
                "UIButton",
                "UISlider",
                "UIToggle",
                "VariableTrigger"
            }));
        }

        public void HandleLoadPreset(string path)
        {
            string aJSON = SuperController.singleton.ReadFileIntoString(path);
            JSONNode jSONNode = JSON.Parse(aJSON);

            str1 = jSONNode["CatList"].AsArray.ToString().Replace("\"", "").Replace("[", "").Replace("]", "").Replace(" ", "");
            var arr1 = str1.Split(',');

            CatList = new List<string>(arr1);

            AtomList.Clear();
            str1 = jSONNode["AtomList"].AsArray.ToString().Replace("\"", "").Replace("[", "").Replace(" ", "");

            foreach (string dummy in CatList)
            {
                if (str1.IndexOf("],") != -1)
                {
                    str2 = str1.Substring(0, str1.IndexOf("],"));
                    str1 = str1.Substring(str1.IndexOf("],") + 2);
                }
                else
                {
                    str2 = str1.Substring(0, str1.Length - 2);
                }
                arr1 = str2.Split(',');
                AtomList.Add(new List<string>(arr1));
            }
        }

        public void FixedUpdate()
        {
            if (sphere != null)
            {
                Destroy(sphere);
            }
            if (sphereY != null)
            {
                Destroy(sphereY);
            }
            if (sphereG != null)
            {
                Destroy(sphereG);
            }
            if (line != null)
            {
                Destroy(line);
            }
            if (line2 != null)
            {
                Destroy(line2);
            }

            if (mark_Bool.val == true)
            {
                if (mkUpDown == true)
                {
                    mkSize += 0.012f;
                    if (mkSize >= 0.5f)
                    {
                        mkUpDown = false;
                    }
                }
                else
                {
                    mkSize -= 0.012f;
                    if (mkSize <= 0.1f)
                    {
                        mkUpDown = true;
                    }
                }

                if (targetJSON.val != "None")
                {
                    targetAtom = SuperController.singleton.GetAtomByUid(targetJSON.val);
                    targetAtomPosition = targetAtom.GetStorableByID("control").transform.position;

                    sphere = GameObject.CreatePrimitive(PrimitiveType.Sphere);
                    sphere.transform.position = targetAtomPosition;
                    sphere.transform.localScale = new Vector3(mkSize, mkSize, mkSize);
                    sphere.GetComponent<Collider>().isTrigger = true;
                    sphere.GetComponent<Renderer>().material.shader = Shader.Find("Battlehub/RTGizmos/Handles");
                    sphere.GetComponent<Renderer>().material.color = Color.magenta;
                }

                if (cr_Bool.val || arrivalJSON.val == "Player")
                {
                    currentAtomPosition = SuperController.singleton.lookCamera.transform.position;
                }
                else if (abp_Bool.val || arrivalJSON.val == "Zero")
                {
                    currentAtomPosition = new Vector3(0f, 0f, 0f);
                    currentAtomRotation = Quaternion.Euler(Vector3.zero);
                }
                else
                {
                    currentAtom = SuperController.singleton.GetAtomByUid(arrivalJSON.val);
                    currentAtomPosition = currentAtom.GetStorableByID("control").transform.position;
                    currentAtomRotation = currentAtom.GetStorableByID("control").transform.rotation;
                }
                sphereG = GameObject.CreatePrimitive(PrimitiveType.Sphere);
                sphereG.transform.position = currentAtomPosition;
                sphereG.transform.localScale = new Vector3(mkSize, mkSize, mkSize);
                sphereG.GetComponent<Collider>().isTrigger = true;
                sphereG.GetComponent<Renderer>().material.shader = Shader.Find("Battlehub/RTGizmos/Handles");
                sphereG.GetComponent<Renderer>().material.color = Color.cyan;
                if (abp_Bool.val)
                {
                    Destroy(sphereG);
                }
                Vector3 targetDistance;
                targetDistance = targetAtomPosition - currentAtomPosition;
                kh = targetAtomPosition;
                float maxDistance = Mathf.Max(Mathf.Abs(targetDistance.x), Mathf.Abs(targetDistance.y), Mathf.Abs(targetDistance.z));

                if (targetJSON.val == "None")
                {
                    if (abp_Bool.val)
                    {
                        bringAtomPosition = Vector3.zero;
                    }
                }
                else
                {
                    if (abp_Bool.val)
                    {
                        bringAtomPosition = Vector3.zero;
                    }
                    else
                    {
                        if (maxDistance > maxD.val)
                        {
                            targetDistance.x = (targetDistance.x / maxDistance) * maxD.val;
                            targetDistance.y = (targetDistance.y / maxDistance) * maxD.val;
                            targetDistance.z = (targetDistance.z / maxDistance) * maxD.val;
                            bringAtomPosition = currentAtomPosition + targetDistance;
                        }
                        else
                        {
                            bringAtomPosition = targetAtomPosition;
                        }
                    }
                    if (kh_Bool.val)
                    {
                        bringAtomPosition.y = kh.y;
                    }
                }

                if (abp_Bool.val || maxD.val == 0f || targetJSON.val != "None")
                {
                    sphereY = GameObject.CreatePrimitive(PrimitiveType.Sphere);
                    sphereY.transform.position = bringAtomPosition;
                    sphereY.transform.localScale = new Vector3(mkSize, mkSize, mkSize);
                    sphereY.GetComponent<Collider>().isTrigger = true;
                    sphereY.GetComponent<Renderer>().material.shader = Shader.Find("Battlehub/RTGizmos/Handles");
                    sphereY.GetComponent<Renderer>().material.color = Color.yellow;
                }

                if (targetJSON.val != "None")
                {
                    line = new GameObject();
                    var rd = line.AddComponent<LineRenderer>();
                    var positions = new Vector3[] {
                        currentAtomPosition,
                        targetAtomPosition
                    };
                    rd.startWidth = 0.02f;
                    rd.endWidth = 0.02f;
                    rd.SetPositions(positions);
                    rd.GetComponent<Renderer>().material.shader = Shader.Find("Battlehub/RTGizmos/Handles");
                    rd.GetComponent<Renderer>().material.color = Color.magenta;
                    rd.alignment = LineAlignment.Local;

                    line2 = new GameObject();
                    var rd2 = line2.AddComponent<LineRenderer>();
                    rd2.startWidth = 0.02f;
                    rd2.endWidth = 0.02f;
                    rd2.SetPositions(positions);
                    rd2.GetComponent<Renderer>().material.shader = Shader.Find("Battlehub/RTGizmos/Handles");
                    rd2.GetComponent<Renderer>().material.color = Color.magenta;
                    rd2.alignment = LineAlignment.View;
                }
            }
        }

        public void OnEnable()
        {
            try
            {
                //SuperController.LogMessage($"{nameof(AddAtom)} enabled");
            }
            catch (Exception e)
            {
                SuperController.LogError($"{nameof(AddAtom)}.{nameof(OnEnable)}: {e}");
            }
        }

        public void OnDisable()
        {
            try
            {
                if (sphere != null)
                {
                    Destroy(sphere);
                }
                if (sphereY != null)
                {
                    Destroy(sphereY);
                }
                if (sphereG != null)
                {
                    Destroy(sphereG);
                }
                if (line != null)
                {
                    Destroy(line);
                }
                if (line2 != null)
                {
                    Destroy(line2);
                }
                //SuperController.LogMessage($"{nameof(AddAtom)} disabled");
            }
            catch (Exception e)
            {
                SuperController.LogError($"{nameof(AddAtom)}.{nameof(OnDisable)}: {e}");
            }
        }

        public void OnDestroy()
        {
            try
            {
                if (sphere != null)
                {
                    Destroy(sphere);
                }
                if (sphereY != null)
                {
                    Destroy(sphereY);
                }
                if (sphereG != null)
                {
                    Destroy(sphereG);
                }
                if (line != null)
                {
                    Destroy(line);
                }
                if (line2 != null)
                {
                    Destroy(line2);
                }
                //SuperController.LogMessage($"{nameof(AddAtom)} destroyed");
            }
            catch (Exception e)
            {
                SuperController.LogError($"{nameof(AddAtom)}.{nameof(OnDestroy)}: {e}");
            }
        }
    }
}
